/************************************************************************
 *
 *                    CalC (Calcium Calculator)
 *                         Victor Matveev
 *
 *                            box.h
 *
 *               Geometry definition classes:
 * 
 *               Generalized box class BoxObj
 *               Box as an obstacle: class ObstBoxObj
 *               Set of boxes and obstacles: RegionObj
 *
 ************************************************************************/

#ifndef CALC_BOX_H_included
#define CALC_BOX_H_included

const int  SURF_XMIN = 1;
const int  SURF_XMAX = 2;
const int  SURF_YMIN = 4;
const int  SURF_YMAX = 8;
const int  SURF_ZMIN = 16;
const int  SURF_ZMAX = 32;
const int  SURF_MASK = 63;
const int  _INSIDE_  = 64;
const int  INSIDE_MASK = 127;

//***************************************************************
//*                   C L A S S   B O X
//***************************************************************

class GridObj;

class BoxObj
{
protected:
  double xmin, xmax, ymin, ymax, zmin, zmax;
  static GridObj *Grid;

public:

  static void bindToGrid(GridObj &grid) { Grid = &grid; }

  void set (double x0, double x1, double y0, double y1, double z0, double z1)
    {xmin=x0; xmax=x1; ymin=y0; ymax=y1; zmin=z0; zmax=z1;}

  void set (const BoxObj &B) { set(B.xmin, B.xmax, B.ymin, B.ymax, B.zmin, B.zmax); }

  BoxObj &operator=(const BoxObj &B) { set(B); return *this; }

  BoxObj(double x0, double x1, double y0, double y1, double z0, double z1) { set(x0, x1, y0, y1, z0, z1); }
  BoxObj()  {};
  ~BoxObj() {};

  virtual int inside(int, int, int);
  int point_type(int, int, int);

};

//***************************************************************

class ObstBoxObj: public BoxObj
{
public:
  ObstBoxObj(double x0, double x1, double y0, double y1, double z0, double z1):
      BoxObj(x0, x1, y0, y1, z0, z1) { };

  ObstBoxObj() : BoxObj() {};

  int inside(int, int, int);
};

//***************************************************************
//*                   C L A S S   R E G I O N
//***************************************************************

class RegionObj
{
protected: 

 double *xgrid, *ygrid, *zgrid;
 double *xcoord, *ycoord, *zcoord;

 int   xsize, ysize, zsize;
 long  xysize;

 int encl_num, obst_num, total;
 int encl_cnt, obst_cnt;

 double xmax, xmin, ymax, ymin, zmax, zmin;

 static GridObj *Grid;

public:

 static void bindToGrid(GridObj &grid) { Grid = &grid; BoxObj::bindToGrid(grid); }

 BoxObj *enclosures;
 ObstBoxObj *obstacles;

 double get_xmax() { return xmax; };
 double get_xmin() { return xmin; };
 double get_ymax() { return ymax; };
 double get_ymin() { return ymin; };
 double get_zmin() { return zmin; };
 double get_zmax() { return zmax; };

 int    get_encl_num() { return encl_num; };
 int    get_obst_num() { return obst_num; };
 int    get_box_num()  { return total; }
 int    get_surface_num()  { return 6 * total; }

 void initialize(int vNum, int oNum);
 RegionObj(int encls, int obstcls) { initialize(encls, obstcls); };
 RegionObj(TokenString &);
 ~RegionObj();

 void add_enclosure(double, double, double, double, double, double);
 void add_enclosure(TokenString &);
 void add_obstacle(double, double, double, double, double, double);
 void add_obstacle(TokenString &);
 int  point_type(int, int, int, int extObstNum, class ObstBoxObj *OBO);
 int  point_type(long, int extObstNum, class ObstBoxObj *OBO);
};

void print_type(int x);


#endif
